// $Id$
//
// Copyright (C) 2006 

#ifndef HH_HTTPConnection_
#define HH_HTTPConnection_ 1

// Custom includes
#include <boost/utility.hpp>
#include <boost/intrusive_ptr.hpp>
#include <boost/scoped_ptr.hpp>
#include "Socket/ClientSocketHandle.hh"
#include "Utils/intrusive_refcount.hh"
#include "Scheduler/ReadHelper.hh"
#include "Scheduler/WriteHelper.hh"
#include "Socket/Protocols/INet/INetAddressing.hh"
#include "HTTPLogger.hh"
#include "HTTPRequest.hh"

//#include "HTTPConnection.mpp"
///////////////////////////////hh.p////////////////////////////////////////

namespace g0dil {
namespace mediaserv {

    class SimpleHTTPServer;
    class StreamConnection;

    class HTTPConnection
	: public senf::intrusive_refcount
    {
    public:
        ///////////////////////////////////////////////////////////////////////////
        // Types

	typedef boost::intrusive_ptr<HTTPConnection> ptr;
	typedef senf::ClientSocketHandle<
	    senf::MakeSocketPolicy<senf::ConnectedCommunicationPolicy,
					  senf::StreamFramingPolicy,
					  senf::ReadablePolicy,
					  senf::WriteablePolicy,
				          senf::INet4AddressingPolicy
				   >::policy> ClientHandle;

	static const unsigned MaxRequestSize = 16384;
	static const unsigned StreamBufferMSecs = 4000;

        ///////////////////////////////////////////////////////////////////////////
        ///\name Structors and default members
        ///@{

        HTTPConnection(ClientHandle client, SimpleHTTPServer & server, HTTPLogger & logger);
	~HTTPConnection();

        ///@}
        ///////////////////////////////////////////////////////////////////////////

	void done();

    protected:

    private:
	void handleRequest(senf::ReadHelper<ClientHandle>::ptr);
	void startStream(senf::WriteHelper<ClientHandle>::ptr);
	
	ClientHandle client_;
	SimpleHTTPServer & server_;
	HTTPLogger & logger_;
	HTTPRequest request_;
	boost::scoped_ptr<StreamConnection> connection_;
	int fileFd_;
	unsigned bandwidth_;
    };

}}

///////////////////////////////hh.e////////////////////////////////////////
//#include "HTTPConnection.cci"
//#include "HTTPConnection.ct"
//#include "HTTPConnection.cti"
//#include "HTTPConnection.mpp"
#endif


// Local Variables:
// mode: c++
// End:
