// $Id$
//
// Copyright (C) 2006 

#ifndef HH_StreamConnection_
#define HH_StreamConnection_ 1

// Custom includes
#include <boost/utility.hpp>
#include <boost/intrusive_ptr.hpp>

#include "Utils/MicroTime.hh"
#include "Utils/intrusive_refcount.hh"
#include "Scheduler/Scheduler.hh"
#include "Socket/ClientSocketHandle.hh"
#include "Socket/CommunicationPolicy.hh"
#include "Socket/FramingPolicy.hh"
#include "Socket/ReadWritePolicy.hh"

//#include "StreamConnection.mpp"
///////////////////////////////hh.p////////////////////////////////////////

namespace g0dil {
namespace mediaserv {

    class StreamConnection : public senf::intrusive_refcount
    {
    public:
        ///////////////////////////////////////////////////////////////////////////
        // Types

	typedef senf::ClientSocketHandle<
	    senf::MakeSocketPolicy<senf::ConnectedCommunicationPolicy,
				    	  senf::StreamFramingPolicy,
				    	  senf::WriteablePolicy>::policy> ClientHandle;

	typedef boost::intrusive_ptr<StreamConnection> ptr;
	typedef boost::function<void ()> Callback;

	static const unsigned packetSize = 1400;

        ///////////////////////////////////////////////////////////////////////////
        ///\name Structors and default members
        ///@{

        StreamConnection(int fileFd, unsigned bytesPerSecond, ClientHandle client,
			 unsigned bufferMSecs, Callback callback);
        ~StreamConnection();

        ///@}
        ///////////////////////////////////////////////////////////////////////////

	void start();
	unsigned bytesSent();

    protected:

    private:
	void registerCallback();
	void unregisterCallback();
	void registerTimeout(unsigned timeout);
	unsigned targetBytes();
	void fillBuffer();
	
	void callback(ClientHandle client, senf::Scheduler::EventId);
	void timeout();
	

	int fileFd_;
	unsigned bytesPerSecond_;
	ClientHandle client_;
	unsigned bufferMSecs_;
	Callback callback_;

	unsigned bytesWritten_;
	senf::MicroTime start_;
	unsigned bufferSize_;
	char buffer_[packetSize];
	bool terminate_;
    };

}}

///////////////////////////////hh.e////////////////////////////////////////
#include "StreamConnection.cci"
//#include "StreamConnection.ct"
//#include "StreamConnection.cti"
//#include "StreamConnection.mpp"
#endif


// Local Variables:
// mode: c++
// End:
